procedure muse_linemap(musecube)
#=====================================================================================================
#This task takes the science stack of the muse datacube and returns a continuum-subtracted line map
#at the rest wavelength of a line. Calculates the observed Doppler-shifted line wavelength and the 
#bandwidth to cover the line over the entire image using the system+/-rotation velocity.
#Continuum is measured as a median over "contwid" on either side of the line wavelength
#centered at +/-(line_bandwidth-contwid/2). Optionally an image section can be analyzed.
#Output line fluxes are in erg/cm2/s units.
#Outputs:
#am644_741_6563 = continuum-subtracted line map
#am644_741_6563_ew = Equivalent width map
#am644_741_6563_cont = off-band continuum map
#am644_741_6563_band = off-band continuum map
#am644_741_6563_cont_sub = difference between blue and red continuum maps
#=====================================================================================================
#
string  musecube  = "am644_741_muse_sci"     	{prompt="Input muse datacube to measure flux"}
string  ofile     = "am644_741_"     		{prompt="Input muse datacube to measure flux"}
real    linelam   = 6562.8                   	{prompt="Rest wavelength of the line in Angstroms"}
real    contwid   = 100.0                    	{prompt="Window width in Angstroms for evaluating continuum"}
real    sysvel    = 6611.                    	{prompt="Systemic Velocity in km/s"}
real    rotvel    = 350.                     	{prompt="Systemic Velocity in km/s"}
int     x1        = 0                        	{prompt="xmin of the subimage"}
int     x2        = 0                        	{prompt="xmax of the subimage"}
int     y1        = 0                        	{prompt="ymin of the subimage"}
int     y2        = 0                        	{prompt="ymax of the subimage"}

begin
	string 	fmusecube, froot, file_line, file_cont_blu, file_cont_red
	string	file_line_2d, ctype, imsec, file_line_final, file_cont_final
	real	cpix, clam, dlam, sys_vel, c_kms, rot_vel
	real	lam_beg, lam_end, lam_lab, rms 
	int	naxis, nx, ny, nlam, ix1, ix2, iy1, iy2, il1, il2
	int	nx0, ny0, npix_line, npix_cont, il1_blu, il2_blu, il1_red, il2_red
	bool	blu_cont, red_cont

	date
	blu_cont = no; red_cont = no
	#fmusecube = "cartwheel_muse_3d_ori.fits"
	#fmusecube = "am644_741_muse.fits"
	fmusecube = musecube   #"am644_741_muse_sci"
	c_kms = 2.9979E5  #velocity of light in km/s
	lam_lab = linelam   #6562.8
	froot = ofile     #"am644_"
	#file_line     = froot//"line" 
	file_cont_blu = froot//"cont_blu" 
	file_cont_red = froot//"cont_red" 
	file_line_2d  = froot//"line_2d"
	file_line_final = froot//str(nint(lam_lab))
	file_cont_final = file_line_final//"_cont"
	file_line       = file_line_final//"_band"

	sys_vel = sysvel   #6611.0
	rot_vel = rotvel   #350.0

	hselect (fmusecube, "NAXIS NAXIS1 NAXIS2 NAXIS3", yes) | scan (naxis, nx, ny, nlam)
	hselect (fmusecube, "CTYPE3 CRPIX3 CRVAL3 CD3_3", yes) | scan (ctype, cpix, clam, dlam)
	print (naxis, nx, ny, nlam)
	print (ctype, " ", cpix, clam, dlam)
	#ix1 = 1; ix2 = nx; iy1 = 1; iy2 = ny
	#ix1 = 280; ix2 = 306; iy1 = 130; iy2 = 160

	ix1 = x1; ix2 = x2; iy1 = y1; iy2 = y2
        if (ix1 <= 0 || ix1 > ix2){ix1 = 1}; if (iy1 <= 0 || iy1 > iy2){iy1 = 1}
        if (ix2 <= ix1 || ix2 > nx){ix2 = nx}; if (iy2 <= iy1 || iy2 > ny){iy2 = ny}
        nx0 = ix2 - ix1 + 1; ny0 = iy2 - iy1 + 1

	lam_beg  = lam_lab*(1.0 + (sys_vel-rot_vel)/c_kms)
	lam_end  = lam_lab*(1.0 + (sys_vel+rot_vel)/c_kms)

	il1 = (lam_beg - clam)/dlam + cpix
	il2 = (lam_end - clam)/dlam + cpix
	if (il1 < 1){il1 = 1}
	if (il2 > nlam){il1 = nlam}
        imsec = "["//str(ix1)//":"//str(ix2)//","//str(iy1)//":"//str(iy2)//","//str(il1)//":"//str(il2)//"]"
	#print (imsec)
        #imdelete(file_line, ver-, >& "dev$null")
	#imcopy (fmusecube//imsec, file_line)
        imdelete(file_line_2d, ver-, >& "dev$null")
        delete("imcombine.log", ver-, >& "dev$null")
	#imcombine (file_line, file_line_2d, combine="sum", project+, logfile="imcombine.log")
	imcombine (fmusecube//imsec, file_line_2d, combine="sum", project+, logfile="imcombine.log")
	imstat (file_line_2d, fields = "image,npix,mean,stddev,min,max,midpt")

	npix_cont = nint(contwid/dlam)
	npix_line = il2 - il1 + 1

	il2_blu = il1 - npix_line 
	il1_blu = il2_blu - npix_cont + 1
	if (il1_blu > 0){
        imsec = "["//str(ix1)//":"//str(ix2)//","//str(iy1)//":"//str(iy2)//","//str(il1_blu)//":"//str(il2_blu)//"]"
        #print (imsec)
        imdelete(file_cont_blu//"_2d", ver-, >& "dev$null")
        delete("imcombine.log", ver-, >& "dev$null")
        imcombine (fmusecube//imsec, file_cont_blu//"_2d", combine="median", project+, logfile="imcombine.log")
	imstat (file_cont_blu//"_2d", fields = "image,npix,mean,stddev,min,max,midpt")
	blu_cont = yes
	}

        il1_red = il2 + npix_line
        il2_red = il1_red + npix_cont - 1
	if (il2_red <= nlam){
        imsec = "["//str(ix1)//":"//str(ix2)//","//str(iy1)//":"//str(iy2)//","//str(il1_red)//":"//str(il2_red)//"]"
        #print (imsec)
        imdelete(file_cont_red//"_2d", ver-, >& "dev$null")
        delete("imcombine.log", ver-, >& "dev$null")
        imcombine (fmusecube//imsec, file_cont_red//"_2d", combine="median", project+, logfile="imcombine.log")
	imstat (file_cont_red//"_2d", fields = "image,npix,mean,stddev,min,max,midpt")
	red_cont = yes
	}
	
        imdelete(file_cont_final, ver-, >& "dev$null")
	if (blu_cont && red_cont){
	  print ("Using average of blue and red continua")
	  imexpr ("1.0E-20*0.5*(a+b)", file_cont_final, file_cont_blu//"_2d", file_cont_red//"_2d", verb-)
	  imstat (file_cont_final, fields = "image,npix,mean,stddev,min,max,midpt")
	  imstat (file_cont_final, fields = "stddev", nclip=9, lsigma=1.0, usigma=1.0, format-) | scan (rms)
          imdelete(file_cont_final//"_sub", ver-, >& "dev$null")
	  imexpr ("1.0E-20*(a-b)", file_cont_final//"_sub", file_cont_blu//"_2d", file_cont_red//"_2d", verb-)
	  imstat (file_cont_final//"_sub", fields = "image,npix,mean,stddev,min,max,midpt")
	} else if (blu_cont) {
	  print ("Using only blue continuum")
	  imstat (file_cont_blu//"_2d", fields = "stddev", nclip=3, lsigma=1.0, usigma=1.0, format-) | scan (rms)
	  imexpr ("1.0E-20*a", file_cont_final, file_cont_blu//"_2d", verb-)
	} else {
	  print ("Using only red continuum")
	  imstat (file_cont_red//"_2d", fields = "stddev", nclip=3, lsigma=1.0, usigma=1.0, format-) | scan (rms)
	  imexpr ("1.0E-20*a", file_cont_final, file_cont_red//"_2d", verb-)
	}
	print ("rms of continuum = ", rms)

        imdelete(file_line, ver-, >& "dev$null")
	imexpr ("1.0E-20*a", file_line, file_line_2d, verb-)
        imdelete(file_line_final, ver-, >& "dev$null")
	#imexpr ("1.0E-20*a-b*c", file_line_final, file_line_2d, file_cont_final, npix_line, verb-)
	imexpr ("(a-b*c)*d", file_line_final, file_line, file_cont_final, npix_line, dlam, verb-)
        imdelete(file_line_final//"_ew", ver-, >& "dev$null")
	imexpr ("a>3.0*b ? c/a : 0.0", file_line_final//"_ew", file_cont_final, rms, file_line_final, verb-)
	date

        imdelete(file_line_2d, ver-, >& "dev$null")
        imdelete(file_cont_blu, ver-, >& "dev$null")
        imdelete(file_cont_red, ver-, >& "dev$null")
        imdelete(file_cont_blu//"_2d", ver-, >& "dev$null")
        imdelete(file_cont_red//"_2d", ver-, >& "dev$null")

end
